@@ -22,7 +22,7 @@ struct Table_connector_s
 {
 	short            lw, rw;
 	Connector        *le, *re;
-	short            cost;
+	unsigned short   cost;
 	s64              count;
 	Table_connector  *next;
 };
@@ -35,14 +35,14 @@ struct count_context_s
 #endif /* USE_FAT_LINKAGES */
 	Word *  local_sent;
 	int     null_block;
-	int     islands_ok;
-	int     null_links;
+	Boolean islands_ok;
+	Boolean null_links;
 	int     table_size;
 	int     log2_table_size;
 	Table_connector ** table;
 	Resources current_resources;
 	Boolean exhausted;
-	int     checktimer;
+	int     checktimer;  /* Avoid excess system calls */
 };
 
 static void free_table(count_context_t *ctxt)
@@ -65,7 +65,7 @@ static void free_table(count_context_t *ctxt)
 
 static void init_table(Sentence sent)
 {
-	int shift;
+	unsigned int shift;
 	/* A piecewise exponential function determines the size of the
 	 * hash table. Probably should make use of the actual number of
 	 * disjuncts, rather than just the number of words.
@@ -85,7 +85,7 @@ static void init_table(Sentence sent)
 
 	/* Clamp at max 4*(1<<24) == 64 MBytes */
 	if (24 < shift) shift = 24;
-	ctxt->table_size = (1 << shift);
+	ctxt->table_size = (1U << shift);
 	ctxt->log2_table_size = shift;
 	ctxt->table = (Table_connector**) 
 		xalloc(ctxt->table_size * sizeof(Table_connector*));
@@ -237,10 +237,10 @@ int do_match(Sentence sent, Connector *a, Connector *b, int aw, int bw)
 static Table_connector * table_store(count_context_t *ctxt,
                                      int lw, int rw,
                                      Connector *le, Connector *re,
-                                     int cost, s64 count)
+                                     unsigned int cost, s64 count)
 {
 	Table_connector *t, *n;
-	int h;
+	unsigned int h;
 
 	n = (Table_connector *) xalloc(sizeof(Table_connector));
 	n->count = count;
@@ -257,10 +257,10 @@ static Table_connector *
 find_table_pointer(count_context_t *ctxt,
                    int lw, int rw, 
                    Connector *le, Connector *re,
-                   int cost)
+                   unsigned int cost)
 {
 	Table_connector *t;
-	int h = pair_hash(ctxt->log2_table_size,lw, rw, le, re, cost);
+	unsigned int h = pair_hash(ctxt->log2_table_size,lw, rw, le, re, cost);
 	t = ctxt->table[h];
 	for (; t != NULL; t = t->next) {
 		if ((t->lw == lw) && (t->rw == rw) && (t->le == le) && (t->re == re)
@@ -286,7 +286,7 @@ find_table_pointer(count_context_t *ctxt,
 
 /** returns the count for this quintuple if there, -1 otherwise */
 s64 table_lookup(Sentence sent, 
-                 int lw, int rw, Connector *le, Connector *re, int cost)
+                 int lw, int rw, Connector *le, Connector *re, unsigned int cost)
 {
 	Table_connector *t = find_table_pointer(sent->count_ctxt, lw, rw, le, re, cost);
 
@@ -314,7 +314,7 @@ static void table_update(count_context_t *ctxt, int lw, int rw,
  * with a count value of 0.
  */
 static s64 pseudocount(Sentence sent,
-                       int lw, int rw, Connector *le, Connector *re, int cost)
+                       int lw, int rw, Connector *le, Connector *re, unsigned int cost)
 {
 	s64 count;
 	count = table_lookup(sent, lw, rw, le, re, cost);
@@ -327,7 +327,8 @@ static s64 do_count(Sentence sent, int lw, int rw,
 	s64 total;
 	int start_word, end_word, w;
 	s64 leftcount, rightcount, pseudototal;
-	int lcost, rcost, Lmatch, Rmatch;
+	Boolean Lmatch, Rmatch;
+	unsigned int lcost, rcost;
 
 	Match_node * m, *m1;
 	Table_connector *t;
@@ -430,9 +431,11 @@ static s64 do_count(Sentence sent, int lw, int rw,
 		m1 = m = form_match_list(sent, w, le, lw, re, rw);
 		for (; m != NULL; m = m->next)
 		{
+			unsigned int null_count_p1;
 			Disjunct * d;
 			d = m->d;
-			for (lcost = 0; lcost <= null_count; lcost++)
+			null_count_p1 = null_count + 1; /* avoid gcc warning: unsafe loop opt */
+			for (lcost = 0; lcost < null_count_p1; lcost++)
 			{
 				rcost = null_count - lcost;
 				/* Now lcost and rcost are the costs we're assigning
