@@ -33,7 +33,7 @@ static Parse_set * dummy_set(void)
 }
 
 /** Returns an empty set of parses */
-static Parse_set * empty_set(void)
+static inline Parse_set * empty_set(void)
 {
 	Parse_set *s;
 	s = (Parse_set *) xalloc(sizeof(Parse_set));
@@ -147,7 +147,8 @@ Parse_info parse_info_new(int nwords)
  */
 void free_parse_info(Parse_info pi)
 {
-	int i, len;
+	unsigned int i;
+	int len;
 	X_table_connector *t, *x;
 
 	len = pi->N_words;
@@ -180,7 +181,7 @@ void free_parse_info(Parse_info pi)
  * Returns the pointer to this info, NULL if not there.
  */
 static X_table_connector * x_table_pointer(int lw, int rw, Connector *le, Connector *re,
-									int cost, Parse_info pi)
+									unsigned int cost, Parse_info pi)
 {
 	X_table_connector *t;
 	t = pi->x_table[pair_hash(pi->log2_x_table_size, lw, rw, le, re, cost)];
@@ -192,7 +193,7 @@ static X_table_connector * x_table_pointer(int lw, int rw, Connector *le, Connec
 
 #if DEAD_CODE
 Parse_set * x_table_lookup(int lw, int rw, Connector *le, Connector *re,
-						   int cost, Parse_info pi) {
+						   unsigned int cost, Parse_info pi) {
 	/* returns the count for this quintuple if there, -1 otherwise */
 	X_table_connector *t = x_table_pointer(lw, rw, le, re, cost, pi);
 
@@ -204,13 +205,13 @@ Parse_set * x_table_lookup(int lw, int rw, Connector *le, Connector *re,
  * Stores the value in the x_table.  Assumes it's not already there.
  */
 static X_table_connector * x_table_store(int lw, int rw, Connector *le, Connector *re,
-								  int cost, Parse_set * set, Parse_info pi)
+								  unsigned int cost, Parse_info pi)
 {
 	X_table_connector *t, *n;
-	int h;
+	unsigned int h;
 
 	n = (X_table_connector *) xalloc(sizeof(X_table_connector));
-	n->set = set;
+	n->set = empty_set();
 	n->lw = lw; n->rw = rw; n->le = le; n->re = re; n->cost = cost;
 	h = pair_hash(pi->log2_x_table_size, lw, rw, le, re, cost);
 	t = pi->x_table[h];
@@ -221,7 +222,7 @@ static X_table_connector * x_table_store(int lw, int rw, Connector *le, Connecto
 
 #ifdef UNUSED_FUNCTION
 static void x_table_update(int lw, int rw, Connector *le, Connector *re,
-					int cost, Parse_set * set, Parse_info pi) {
+					unsigned int cost, Parse_set * set, Parse_info pi) {
 	/* Stores the value in the x_table.  Unlike x_table_store, it assumes it's already there */
 	X_table_connector *t = x_table_pointer(lw, rw, le, re, cost, pi);
 
@@ -240,12 +241,13 @@ static void x_table_update(int lw, int rw, Connector *le, Connector *re,
  */
 static Parse_set * parse_set(Sentence sent,
                  Disjunct *ld, Disjunct *rd, int lw, int rw,
-					  Connector *le, Connector *re, int cost,
-                 int islands_ok, Parse_info pi)
+					  Connector *le, Connector *re, unsigned int cost,
+                 Boolean islands_ok, Parse_info pi)
 {
 	Disjunct * d, * dis;
 	int start_word, end_word, w;
-	int lcost, rcost, Lmatch, Rmatch;
+	Boolean Lmatch, Rmatch;
+	unsigned int lcost, rcost;
 	int i, j;
 	Parse_set *ls[4], *rs[4], *lset, *rset;
 	Parse_choice * a_choice;
@@ -254,7 +256,7 @@ static Parse_set * parse_set(Sentence sent,
 	X_table_connector *xt;
 	s64 count;
 
-	assert(cost >= 0, "parse_set() called with cost < 0.");
+	assert(cost < 0x7fff, "parse_set() called with cost < 0.");
 
 	count = table_lookup(sent, lw, rw, le, re, cost);
 
@@ -273,7 +275,7 @@ static Parse_set * parse_set(Sentence sent,
 
 	/* Start it out with the empty set of options. */
 	/* This entry must be updated before we return. */
-	xt = x_table_store(lw, rw, le, re, cost, empty_set(), pi);
+	xt = x_table_store(lw, rw, le, re, cost, pi);
 
 	xt->set->count = count;  /* the count we already computed */
 	/* this count is non-zero */
@@ -444,9 +446,9 @@ static int verify_set_node(Parse_set *set)
 	return FALSE;
 }
 
-static int verify_set(Parse_info pi)
+static Boolean verify_set(Parse_info pi)
 {
-	int i;
+	unsigned int i;
 
 	assert(pi->x_table != NULL, "called verify_set when x_table==NULL");
 	for (i=0; i<pi->x_table_size; i++)
@@ -476,7 +478,7 @@ static int verify_set(Parse_info pi)
  * This routine returns TRUE iff overflowed occurred.
  */
 
-int build_parse_set(Sentence sent, int cost, Parse_Options opts)
+int build_parse_set(Sentence sent, unsigned int cost, Parse_Options opts)
 {
 	Parse_set * whole_set;
 
@@ -616,7 +618,7 @@ static void list_random_links(Parse_info pi, Parse_set * set)
  * sentence.  For this to work, you must have already called parse, and
  * already built the whole_set.
  */
-void extract_links(int index, int cost, Parse_info pi)
+void extract_links(int index, Parse_info pi)
 {
 	initialize_links(pi);
 	pi->rand_state = index;
