@@ -134,12 +134,12 @@ static per_thread_data * init(JNIEnv *env, jclass cls)
 	memset(ptd, 0, sizeof(per_thread_data));
 
 	ptd->opts = parse_options_create();
-	parse_options_set_disjunct_costf(ptd->opts, 3.0f);
-	parse_options_set_max_sentence_length(ptd->opts, 170);
+	parse_options_set_disjunct_costf(ptd->opts, 2.0f);
+	parse_options_set_max_sentence_length(ptd->opts, MAX_SENTENCE-3);
 	parse_options_set_max_parse_time(ptd->opts, 30);
 	parse_options_set_linkage_limit(ptd->opts, 1000);
 	parse_options_set_short_length(ptd->opts, 10);
-	parse_options_set_verbosity(ptd->opts,0);
+	parse_options_set_verbosity(ptd->opts, 0);
 	parse_options_set_spell_guess(ptd->opts, FALSE);
 
 	parse_options_set_panic_mode(ptd->opts, TRUE);
@@ -549,7 +549,7 @@ Java_org_linkgrammar_LinkGrammar_getLinkageSenseScore(JNIEnv *env,
 {
 	per_thread_data *ptd = get_ptd(env, cls);
 	Linkage lkg = ptd->linkage;
-	Linkage_info *lifo = lkg;
+	Linkage_info *lifo = lkg->info;
 	Sense *sns;
 	double score = 0.0;
 
@@ -616,18 +616,6 @@ Java_org_linkgrammar_LinkGrammar_getLinkageNumViolations(JNIEnv *env, jclass cls
 	return sentence_num_violations(ptd->sent, ptd->cur_linkage);
 }
 
-/*
- * Class:      LinkGrammar
- * Method:     linkageAndCost
- * Signature: ()I
- */
-JNIEXPORT jint JNICALL
-Java_org_linkgrammar_LinkGrammar_getLinkageAndCost(JNIEnv *env, jclass cls)
-{
-	per_thread_data *ptd = get_ptd(env, cls);
-	return sentence_and_cost(ptd->sent, ptd->cur_linkage);
-}
-
 /*
  * Class:      LinkGrammar
  * Method:     linkageDisjunctCost
@@ -768,41 +756,3 @@ Java_org_linkgrammar_LinkGrammar_getLinkString(JNIEnv *env, jclass cls)
 	return j;
 }
 
-/*
- * Class:      LinkParser
- * Method:     isPastTenseForm
- * Signature: (Ljava/lang/String;)Z
- *
- * @deprecated -- past-tense verbs are tagged with .v-d or .w-d or .q-d
- * subscripts. use those instead to figure out if a verb is past tense.
- */
-JNIEXPORT jboolean JNICALL
-Java_org_linkgrammar_LinkGrammar_isPastTenseForm(JNIEnv *env, jclass cls, jstring str)
-{
-	jboolean rv = FALSE;
-
-	per_thread_data *ptd = get_ptd(env, cls);
-	const char *cStr = (*env)->GetStringUTFChars(env,str,0);
-	if (dictionary_is_past_tense_form(ptd->dict, cStr) == 1)
-		rv = TRUE;
-	(*env)->ReleaseStringUTFChars(env,str,cStr);
-	return rv;
-}
-
-/*
- * Class:      LinkParser
- * Method:     isEntity
- * Signature: (Ljava/lang/String;)Z
- */
-JNIEXPORT jboolean JNICALL
-Java_org_linkgrammar_LinkGrammar_isEntity(JNIEnv *env, jclass cls, jstring str)
-{
-	jboolean rv = FALSE;
-
-	per_thread_data *ptd = get_ptd(env, cls);
-	const char *cStr = (*env)->GetStringUTFChars(env,str,0);
-	if (dictionary_is_entity(ptd->dict, cStr) == 1)
-		rv = TRUE;
-	(*env)->ReleaseStringUTFChars(env,str,cStr);
-	return rv;
-}
