@@ -12,12 +12,11 @@
 /*************************************************************************/
 
 void      print_time(Parse_Options opts, const char * s);
-/* void      print_total_time(Parse_Options opts); */
 void      print_total_space(Parse_Options opts);
 void      resources_reset(Resources r);
 void      resources_reset_space(Resources r);
-int       resources_timer_expired(Resources r);
-int       resources_memory_exhausted(Resources r);
-int       resources_exhausted(Resources r);
+Boolean   resources_timer_expired(Resources r);
+Boolean   resources_memory_exhausted(Resources r);
+Boolean   resources_exhausted(Resources r);
 Resources resources_create(void); 
 void      resources_delete(Resources ti);
