@@ -1,7 +1,7 @@
 /*************************************************************************/
 /* Copyright (c) 2004                                                    */
 /* Daniel Sleator, David Temperley, and John Lafferty                    */
-/* Copyright (c) 2009, 2012 Linas Vepstas                                */
+/* Copyright (c) 2009, 2012, 2013 Linas Vepstas                          */
 /* All rights reserved                                                   */
 /*                                                                       */
 /* Use of the link grammar parsing system is subject to the terms of the */
@@ -543,12 +543,55 @@ Boolean split_word(Tokenizer *tokenizer, Dictionary dict, const char *word)
 	return suffix_split(tokenizer, dict, word, word + strlen(word));
 }
 
+
+/* Return TRUE if the word might be capitalized by convention:
+ * -- if its the first word of a sentence
+ * -- if its the first word following a colon
+ * -- if its the first word of a quote
+ *
+ * XXX FIXME: These rules are rather English-centric.  Someone should
+ * do something about this someday.
+ */
+static Boolean is_capitalizable(Sentence sent, int curr_word)
+{
+	int first_word; /* the index of the first word after the wall */
+	Dictionary dict = sent->dict;
+
+	if (dict->left_wall_defined) {
+		first_word = 1;
+	} else {
+		first_word = 0;
+	}
+
+	/* Words at the start of sentences are capitalizable */
+	if (curr_word == first_word) return TRUE;
+
+	/* Words following colons are capitalizable */
+	if (curr_word > 0 && strcmp(":", sent->word[curr_word-1].alternatives[0])==0)
+		return TRUE;
+
+	/* First word after a quote mark can be capitalized */
+	if (sent->post_quote[curr_word])
+		return TRUE;
+
+	return FALSE;
+}
+
 /**
  * w points to a string, wend points to the char one after the end.  The
  * "word" w contains no blanks.  This function splits up the word if
  * necessary, and calls "issue_sentence_word()" on each of the resulting
  * parts.  The process is described above.  Returns TRUE if OK, FALSE if
  * too many punctuation marks or other separation error.
+ *
+ * This is used to split Russian words into stem+suffix, issueing a
+ * separate "word" for each.  In addition, there are many English
+ * constructions that need splitting:
+ *
+ * 86mm  -> 86 + mm (millimeters, measureument)
+ * $10   ->  $ + 10 (dollar sign plus a number)
+ * Surprise!  -> surprise + !  (pry the punctuation off the end of the word)
+ * you've   -> you + 've  (undo contraction, treat 've as synonym for 'have')
  */
 static void separate_word(Sentence sent, Parse_Options opts,
                          const char *w, const char *wend,
@@ -600,10 +643,18 @@ static void separate_word(Sentence sent, Parse_Options opts,
 	 * we should be doing it here, and building two alternatives, one
 	 * capitalized, and one not.  This is particularly true for the
 	 * Russian dictionaries, where captialization and suffix splitting
-	 * only 'accidentally' work, due toe CAPITALIZED_WORDS in the regex
+	 * only 'accidentally' work, due to CAPITALIZED_WORDS in the regex
 	 * file, which matches capitalized stems (for all the wrong reasons...)
 	 * Now that we have alternatives, we should use them wisely.
+	 *
+	 * Well, actually, easier said-than-done. The problem is that
+	 * captilized words will match regex's during build_expressions,
+	 * and we sometimes need to wipe those out...
 	 */
+#if 0
+	if (is_capitalizable(sent, sent->length) && is_utf8_upper(word)) {}
+#endif
+
 	if (word_is_in_dict && !have_suffix)
 	{
 		issue_sentence_word(sent, word, quote_found);
@@ -878,11 +929,11 @@ do_suffix_processing:
  */
 Boolean separate_sentence(Sentence sent, Parse_Options opts)
 {
-	const char *t;
+	const char * word_end;
 	Boolean quote_found;
 	Dictionary dict = sent->dict;
 	mbstate_t mbs;
-	const char * s = sent->orig_sentence;
+	const char * word_start = sent->orig_sentence;
 
 	sent->length = 0;
 
@@ -896,7 +947,7 @@ Boolean separate_sentence(Sentence sent, Parse_Options opts)
 	{
 		int isq;
 		wchar_t c;
-		int nb = mbrtowc(&c, s, MB_CUR_MAX, &mbs);
+		int nb = mbrtowc(&c, word_start, MB_CUR_MAX, &mbs);
 		quote_found = FALSE;
 
 		if (0 > nb) goto failure;
@@ -910,29 +961,31 @@ Boolean separate_sentence(Sentence sent, Parse_Options opts)
 		if (isq) quote_found = TRUE;
 		while (iswspace(c) || isq)
 		{
-			s += nb;
-			nb = mbrtowc(&c, s, MB_CUR_MAX, &mbs);
+			word_start += nb;
+			nb = mbrtowc(&c, word_start, MB_CUR_MAX, &mbs);
 			if (0 == nb) break;
 			if (0 > nb) goto failure;
 			isq = is_quote (c);
 			if (isq) quote_found = TRUE;
 		}
 
-		if (*s == '\0') break;
+		if ('\0' == *word_start) break;
 
-		t = s;
-		nb = mbrtowc(&c, t, MB_CUR_MAX, &mbs);
+      /* Loop over non-blank characters until word-end is found. */
+		word_end = word_start;
+		nb = mbrtowc(&c, word_end, MB_CUR_MAX, &mbs);
 		if (0 > nb) goto failure;
 		while (!iswspace(c) && !is_quote(c) && (c != 0) && (nb != 0))
 		{
-			t += nb;
-			nb = mbrtowc(&c, t, MB_CUR_MAX, &mbs);
+			word_end += nb;
+			nb = mbrtowc(&c, word_end, MB_CUR_MAX, &mbs);
 			if (0 > nb) goto failure;
 		}
 
-		separate_word(sent, opts, s, t, quote_found);
-		s = t;
-		if (*s == '\0') break;
+      /* Perform prefix, suffix splitting, if needed */
+		separate_word(sent, opts, word_start, word_end, quote_found);
+		word_start = word_end;
+		if ('\0' == *word_start) break;
 	}
 
 	if (dict->right_wall_defined)
@@ -1099,25 +1152,19 @@ static X_node * guess_misspelled_word(Sentence sent, int i, const char * s)
  * spell-guesser will offer both "done" and "don't" as alternatives.
  * The second alternative should really be affix-stripped, and issued
  * as two words, not one. But the former only issues as one word.  So,
- * should we issue two word or one?  ... and so we can't correctly
+ * should we issue two words, or one?  ... and so we can't correctly
  * build sentence expressions for this, since we don't have the correct
  * word count by this point.  The viterbi incremental parser won't have
  * this bug.
  */
 void build_sentence_expressions(Sentence sent, Parse_Options opts)
 {
-	int i, first_word;  /* the index of the first word after the wall */
+	int i;
 	const char *s;
 	const char * regex_name;
 	X_node * e;
 	Dictionary dict = sent->dict;
 
-	if (dict->left_wall_defined) {
-		first_word = 1;
-	} else {
-		first_word = 0;
-	}
-
 	/* The following loop treats all words the same
 	 * (nothing special for 1st word) */
 	for (i=0; i<sent->length; i++)
@@ -1161,24 +1208,17 @@ void build_sentence_expressions(Sentence sent, Parse_Options opts)
 			 * capitalized has to be looked up as an uncapitalized word
 			 * (possibly also as well as a capitalized word).
 			 *
-			 * XXX This rule is English-language-oriented, and should be
-			 * abstracted.
-			 *
 			 * XXX For the first-word case, we should be handling capitalization
 			 * as an alternative, when doing separate_word(), and not here.
 			 * separate_word() should build capitalized and non-capitalized
 			 * altenatives.  This is especially true for Russian, where we
-			 * need to deal with capitalized stems; thhis is not really the
+			 * need to deal with capitalized stems; this is not really the
 			 * right place to do it, and this works 'by accident' only because
 			 * there is a CAPITALIZED_WORDS regex match for Russian that matches
 			 * stems. Baaaddd.
 			 */
-			if ((i == first_word ||
-			     (i > 0 && strcmp(":", sent->word[i-1].alternatives[0])==0) || 
-			     sent->post_quote[i]) &&
-			   (is_utf8_upper(s)))
+			if (is_capitalizable(sent, i) && is_utf8_upper(s))
 			{
-
 				/* If the lower-case version of this word is in the dictionary, 
 				 * then add the disjuncts for the lower-case version. The upper
 				 * case version disjuncts had previously come from matching the 
@@ -1216,16 +1256,14 @@ void build_sentence_expressions(Sentence sent, Parse_Options opts)
 				 */
 				if (boolean_dictionary_lookup(sent->dict, lc))
 				{
+					if (2 < verbosity)
+						printf ("Info: First word: %s is_entity=%d is_common=%d\n", 
+						        s, is_entity(sent->dict,s),
+						        is_common_entity(sent->dict,lc));
+
 					if (is_entity(sent->dict,s) ||
 					    is_common_entity(sent->dict,lc))
 					{
-						if (2 < verbosity)
-						{
-							printf ("Info: First word: %s entity=%d common=%d\n", 
-								s, is_entity(sent->dict,s), 
-								is_common_entity(sent->dict,lc));
-						}
-	
 						/* If we are here, then we want both upper and lower case
 						 * expressions. The upper-case ones were built above, so now
 						 * append the lower-case ones. */
@@ -1235,9 +1273,7 @@ void build_sentence_expressions(Sentence sent, Parse_Options opts)
 					else
 					{
 						if (2 < verbosity)
-						{
 							printf("Info: First word: %s downcase only\n", lc);
-						}
 	
 						/* If we are here, then we want the lower-case 
 						 * expressions only.  Erase the upper-case ones, built
