@@ -46,7 +46,7 @@ void        exfree_link(Link *);
 /* Connector-set utilities ... */
 Connector_set * connector_set_create(Exp *e);
 void connector_set_delete(Connector_set * conset);
-int match_in_connector_set(Sentence, Connector_set *conset, Connector * c, int d);
+Boolean match_in_connector_set(Sentence, Connector_set *conset, Connector * c, int d);
 Boolean word_has_connector(Dict_node *, const char *, char);
 
 
@@ -141,12 +141,12 @@ static inline int connector_hash(Connector * c)
  * hash function. Based on some tests, this seems to be an almost
  * "perfect" hash, in that almost all hash buckets have the same size!
  */
-static inline int pair_hash(int log2_table_size,
+static inline unsigned int pair_hash(unsigned int log2_table_size,
                             int lw, int rw,
                             const Connector *le, const Connector *re,
-                            int cost)
+                            unsigned int cost)
 {
-	int table_size = (1 << log2_table_size);
+	unsigned int table_size = (1 << log2_table_size);
 	unsigned int i = 0;
 
 #if 0
