@@ -1,5 +1,5 @@
 /*************************************************************************/
-/* Copyright (c) 2012 Linas Vepstas <linasvepstas@gmail.com>             */
+/* Copyright (c) 2012, 2013 Linas Vepstas <linasvepstas@gmail.com>       */
 /* All rights reserved                                                   */
 /*                                                                       */
 /* Use of the Viterbi parsing system is subject to the terms of the      */
@@ -9,21 +9,24 @@
 /*                                                                       */
 /*************************************************************************/
 
-#ifndef _LG_VITERBI_ATOM_H
-#define _LG_VITERBI_ATOM_H
+#ifndef _ATOMBASE_ATOM_H
+#define _ATOMBASE_ATOM_H
 
 #include <iostream>
+#include <mutex>
+#include <set>
 #include <string>
 #include <vector>
+
 #include <gc/gc_allocator.h>
+#include <gc/gc_cpp.h>
 
-#include "garbage.h"
+#include "atom-types.h"
 
-namespace link_grammar {
-namespace viterbi {
+namespace atombase {
 
 // Classes generally resembling those of the OpenCog AtomSpace
-// These are tailored for use for parsing.
+// These are tailored for use for the tracking task.
 
 /**
  * TV (truth value): strength or likelihood of a link.
@@ -59,31 +62,6 @@ class TV
 		}
 };
 
-// Atom types.  Right now an enum, but maybe should be dynamic!?
-enum AtomType
-{
-	// Node types
-	NODE = 1,
-	WORD,       // a word
-	LING_TYPE,  // a pair of merged connectors (LG LINK TYPE)
-	// META,       // special-word, e.g. LEFT-WALL, RIGHT-WALL
-	CONNECTOR,  // e.g. S+ 
-
-	// Link types
-	SET,        // unordered set of children
-	SEQ,        // ordered sequence of children
-	AND,        // ordered AND of all children (order is important!)
-	OR,         // unordered OR of all children
-	WORD_CSET,  // word, followed by a set of connectors for that word.
-	WORD_DISJ,  // word, followed by a single disjunct for that word.
-	LING,       // two connected connectors, (LGLINK) e.g. Dmcn w/o direction info
-	STATE_TRIPLE, // Current pending input, parse state and corresponding output.
-
-	RULE,       // Base class for graph re-write rules
-};
-
-#define OPTIONAL_CLAUSE "0"
-
 /* Base class for Nodes and Links */
 /**
  * Atoms are not mutable, except for the TV value. That is, you cannot
@@ -98,26 +76,65 @@ enum AtomType
  *
  * All atoms are automatically garbage-collected.
  */
+class Link;
+class Relation;
+class Set;
 class Atom : public gc
 {
 	public:
 		Atom(AtomType type, const TV& tv = TV()) :
-			_type(type), _tv(tv)
-		{
-			// Marking stubborn, since its immutable.
-			GC_change_stubborn(this);
-		}
+			_tv(tv), _type(type), _incoming_set(NULL) {}
+		virtual ~Atom();
 		AtomType get_type() const { return _type; }
 		TV _tv;
+
+		void keep_incoming_set();
+		void drop_incoming_set();
+		Set* get_incoming_set() const;
+		Set* get_incoming_set(AtomType) const;
+
+		Relation* add_relation(const std::string&, Atom*);
+		Set* get_relations(const std::string&) const;
+		Set* get_relation_vals(const std::string&) const;
+
 		virtual bool operator==(const Atom*) const;
 		virtual Atom* clone() const = 0;
-		virtual ~Atom() {}
+		Atom* upcaster();
 	protected:
+		friend class Link;  // wtf ???
+		void insert_atom(Link*);
+		void remove_atom(Link*);
+
 		const AtomType _type;
+
+		struct IncomingSet : public gc
+		{
+				// Just right now, we will use a single shared mutex for all
+				// locking on the incoming set.  If this causes too much
+				// contention, then we can fall back to a non-global lock,
+				// at the cost of 40 additional bytes per atom.
+				static std::mutex _mtx;
+				// incoming set is not tracked by garbage collector,
+				// to avoid cyclic references.
+				// std::set<ptr> uses 48 bytes (per atom).
+				std::set<Link*, std::less<Link*>, gc_allocator<Atom*> > _iset;
+		};
+		IncomingSet* _incoming_set;
+
+		Set* filter_iset(std::function<Atom* (Atom*)>) const;
 };
 
+/// Given an atom of a given type, return the C++ class of that type.
+template<typename T>
+T upcast(Atom* a)
+{
+	T t = dynamic_cast<T>(a);
+	if (t) return t;
+	return dynamic_cast<T>(a->upcaster());
+}
+
 /**
- * A Node may be 
+ * A Node may be
  * -- a word (the std::string holds the word)
  * -- a link (the std::string holds the link)
  * -- a disjunct (the std::string holds the disjunct)
@@ -142,10 +159,15 @@ class Node : public Atom
 		const std::string _name;
 };
 
+
 /// All outgoing lists will be handled as vectors.
 // Must use the bdw-gc allocator to track these pointers.
 // If this is not done, the GC will fail to see the pointers here.
-typedef std::vector<Atom*, gc_allocator<Atom*> > OutList;
+template <typename T> 
+using AtomList = std::vector<T, gc_allocator<Atom*> >;
+
+// typedef std::vector<Atom*, gc_allocator<Atom*> > OutList;
+typedef AtomList<Atom*> OutList;
 
 /**
  * Links hold a bunch of atoms
@@ -157,45 +179,71 @@ class Link : public Atom
 	public:
 		// The main ctor
 		Link(AtomType t, const OutList& oset, const TV& tv = TV())
-			: Atom(t, tv), _oset(oset) {}
+			: Atom(t, tv), _oset(oset)
+		{ add_to_incoming_set(); }
 		Link(AtomType t, const TV& tv = TV())
 			: Atom(t, tv)
-		{}
+		{ add_to_incoming_set(); }
 		Link(AtomType t, Atom* a, const TV& tv = TV())
 			: Atom(t, tv), _oset(1, a)
-		{}
+		{ add_to_incoming_set(); }
 		Link(AtomType t, Atom* a, Atom*b, const TV& tv = TV())
 			: Atom(t, tv), _oset(({OutList o(1,a); o.push_back(b); o;}))
-		{}
+		{ add_to_incoming_set(); }
 		Link(AtomType t, Atom* a, Atom* b, Atom* c, const TV& tv = TV())
 			: Atom(t, tv), _oset(({OutList o(1,a); o.push_back(b);
 			                      o.push_back(c); o;}))
-		{}
+		{ add_to_incoming_set(); }
 		Link(AtomType t, Atom* a, Atom* b, Atom* c, Atom* d, const TV& tv = TV())
 			: Atom(t, tv), _oset(({OutList o(1,a); o.push_back(b);
 			                      o.push_back(c); o.push_back(d); o;}))
-		{}
+		{ add_to_incoming_set(); }
 		Link(AtomType t, Atom* a, Atom* b, Atom* c, Atom* d, Atom* e, const TV& tv = TV())
 			: Atom(t, tv), _oset(({OutList o(1,a); o.push_back(b);
 			                      o.push_back(c); o.push_back(d);
 			                      o.push_back(e); o;}))
-		{}
+		{ add_to_incoming_set(); }
+		virtual ~Link();
+
 		size_t get_arity() const { return _oset.size(); }
 		Atom* get_outgoing_atom(size_t pos) const { return _oset.at(pos); }
 		const OutList& get_outgoing_set() const { return _oset; }
 
+		void enable_keep_incoming_set(AtomType);
+		void disable_keep_incoming_set(AtomType);
+		void add_to_incoming_set();
+		void add_to_incoming_set(AtomType);
+		void remove_from_incoming_set(AtomType);
+		Link* append(Atom*) const;
+
 		virtual bool operator==(const Atom*) const;
 		virtual Link* clone() const { return new Link(*this); }
 	protected:
-		// Outgoing set is const, nnot modifiable.
+		// Outgoing set is const, not modifiable.
 		const OutList _oset;
 };
 
+
+// An unhygenic for-each loop, to simplify iterating over
+// the outgoing set.  I don't see a more elegant way to do this,
+// just right now...
+// Anyway, this implements the semantics "foreach VAR of TYPENAME in LNK"
+#define foreach_outgoing(TYPENAME,VAR,LNK) \
+	const atombase::Link* _ll_##VAR; \
+	size_t _ii_##VAR, _ee_##VAR; \
+	atombase::Atom* _aa_##VAR; \
+	TYPENAME VAR; \
+	for (_ll_##VAR = (LNK), _ii_##VAR = 0, \
+	     _ee_##VAR = _ll_##VAR->get_arity(); \
+	     _aa_##VAR = (_ii_##VAR < _ee_##VAR) ? \
+	        _ll_##VAR->get_outgoing_atom(_ii_##VAR) : 0x0, \
+	     VAR = dynamic_cast<TYPENAME>(_aa_##VAR), \
+	     _ii_##VAR < _ee_##VAR; \
+	     _ii_##VAR++)
+
+
 std::ostream& operator<<(std::ostream& out, const Atom*);
-std::ostream& operator<<(std::ostream& out, AtomType);
 
+} // namespace atombase
 
-} // namespace viterbi
-} // namespace link-grammar
-
-#endif // _LG_VITERBI_ATOM_H
+#endif // _ATOMBASE_ATOM_H
