@@ -61,7 +61,7 @@ StateTriple* Connect::try_alternative(Atom* ldj, Atom* rdj)
 		else
 		{
 			// Right disunct better be a multi-connector
-			And* rand = dynamic_cast<And*>(upcast(rdj));
+			And* rand = upcast<And*>(rdj);
 			assert(rand, "Right dj not a disjunct");
 			return alternative(lcon, rand);
 		}
@@ -69,7 +69,7 @@ StateTriple* Connect::try_alternative(Atom* ldj, Atom* rdj)
 	else
 	{
 		// Left disjunct better be a multi-connector.
-		And* land = dynamic_cast<And*>(upcast(ldj));
+		And* land = upcast<And*>(ldj);
 		assert(land, "Left dj not a disjunct");
 
 		// Right disjunct is a single connector
@@ -78,7 +78,7 @@ StateTriple* Connect::try_alternative(Atom* ldj, Atom* rdj)
 		else
 		{
 			// Right disunct better be a multi-connector
-			And* rand = dynamic_cast<And*>(upcast(rdj));
+			And* rand = upcast<And*>(rdj);
 			assert(rand, "Right dj not a disjunct");
 
 			return alternative(land, rand);
@@ -147,7 +147,6 @@ StateTriple* Connect::alternative(Connector* lcon, And* rand)
 
 	Atom* rfirst = rand->get_outgoing_atom(0);
 	Connector* rfc = dynamic_cast<Connector*>(rfirst);
-if (!rfc) cout <<"duuude wttttf "<<rfirst<<endl;
 	assert(rfc, "Exepcting a connector in the right disjunct");
 
 	Ling* conn = conn_connect_nn(lcon, rfc);
@@ -362,7 +361,7 @@ const OutList& Connect::flatten(OutList& alternatives)
 			return set->get_outgoing_set();
 	}
 
-	for (int i = 0; i < asize; i++)
+	for (size_t i = 0; i < asize; i++)
 	{
 		Set* set = dynamic_cast<Set*>(alternatives[i]);
 		if (set and (1 == set->get_arity()))
